#ifndef FIXEDSIZEHEAP_HPP
#define FIXEDSIZEHEAP_HPP

#include "Common.hpp"

template<size_t size>
class FixedSizeHeap
{
public:
    constexpr static size_t allocSize = ((size == 0 ? 1 : size) + 7) & ~(size_t)7;

    FixedSizeHeap(size_t heapSize) :
        m_freeList(),
        m_mem(::malloc(heapSize)),
        m_nextAlloc(m_mem)
    {
    }

    ~FixedSizeHeap()
    {
        ::free(m_mem);
    }

    void* malloc()
    {
        void* p = m_freeList;
        if (p)
        {
            m_freeList = *(void**)p;
        }
        else
        {
            p = m_nextAlloc;
            m_nextAlloc = (char*)m_nextAlloc + allocSize;
        }
        return p;

    }

    void* calloc()
    {
        void* p = malloc();
        memset(p, 0, size);
        return p;
    }

    void free(void* p)
    {
        *(void**)p = m_freeList;
        m_freeList = p;
    }

private:
    void* m_freeList;
    void* m_mem;
    void* m_nextAlloc;
};

#endif
