#include "Problem.hpp"
#include "FileUtils.hpp"
#include "StringUtils.hpp"
#include "ParseUtils.hpp"

using std::string;
using std::vector;

bool readProblem(uint64_t id,
                 string* pProblemData)
{
    string fileName = strprintf("../problems/spaceship%" PRIu64 "", id);
    return readFile(fileName, pProblemData);
}

bool writeProblem(uint64_t id,
                  const string& problemData)
{
    string fileName = strprintf("../problems/spaceship%" PRIu64 "", id);
    return writeFile(fileName, problemData);
}

bool parseProblem(const string& problemData,
                  Problem* pProblem,
                  string* pMsg)
{
    Problem problem;

    vector<string> lines;
    getLines(problemData, &lines);

    for (auto& line : lines)
    {
        if (line.empty()) continue;
        size_t spacePos = line.find(' ');
        if (spacePos == string::npos)
        {
            if (pMsg) *pMsg = "Space not found";
            return false;
        }
        string strX = line.substr(0, spacePos);
        string strY = line.substr(spacePos + 1);

        int64_t x;
        if (!parseI64(strX, &x))
        {
            if (pMsg) *pMsg = "Error parsing X coordinate: '" + strX + "'";
            return false;
        }
        int64_t y;
        if (!parseI64(strY, &y))
        {
            if (pMsg) *pMsg = "Error parsing Y coordinate: '" + strY + "'";
            return false;
        }

        problem.emplace_back(x, y);
    }

    if (pProblem) *pProblem = std::move(problem);
    return true;
}

bool readAndParseProblem(uint64_t id,
                         Problem* pProblem,
                         string* pMsg)
{
    string problemData;
    if (!readProblem(id, &problemData))
    {
        if (pMsg) *pMsg = strprintf("Error reading problem %" PRIu64 "", id);
        return false;
    }

    return parseProblem(problemData, pProblem, pMsg);
}

void formatProblem(const Problem& problem,
                   string* pProblemData)
{
    string problemData;
    for (Coord coord : problem)
    {
        problemData += strprintf("%" PRIi64 " %" PRIi64 "\n", coord.x, coord.y);
    }

    if (pProblemData) *pProblemData = std::move(problemData);
}

bool readSolution(uint64_t id,
                  string* pSolution)
{
    string fileName = strprintf("../solutions/spaceship%" PRIu64 "", id);
    return readFile(fileName, pSolution);
}

bool writeSolution(uint64_t id,
                   const string& solution)
{
    string fileName = strprintf("../solutions/spaceship%" PRIu64 "", id);
    return writeFile(fileName, solution);
}
