#include "Protocol.hpp"
#include "Http.hpp"
#include "FileUtils.hpp"

using std::string;
using std::vector;

namespace Protocol
{
    string apiUrl = "https://boundvariable.space/";

    void init()
    {
        Http::init();
    }

    void cleanup()
    {
        Http::cleanup();
    }

    bool getAPIKey(string* pKey)
    {
        string tmpKey;
        if (!readFile("apikey.txt", &tmpKey) &&
            !readFile("../apikey.txt", &tmpKey) &&
            !readFile("../../apikey.txt", &tmpKey) &&
            !readFile("../../../apikey.txt", &tmpKey))
        {
            return false;
        }

        string key;
        for (char ch : tmpKey)
        {
            if (ch >= 33 && ch <= 126)
            {
                key.push_back(ch);
            }
        }
        if (key.empty())
        {
            return false;
        }

        if (pKey) *pKey = std::move(key);
        return true;
    }

    bool getAPIKeyHeader(vector<string>* pHeaders,
                         std::string* pMsg = nullptr)
    {
        string key;
        if (!getAPIKey(&key))
        {
            if (pMsg) *pMsg = "Error reading API key";
            return false;
        }

        pHeaders->push_back("Authorization: Bearer " + key);
        return true;
    }

    void getUserAgentHeader(vector<string>* pHeaders)
    {
        pHeaders->push_back("User-Agent: libcurl (Frictionless Bananas ICFPC tools)");
    }

    bool communicate(const string& request,
                     string* pResponse,
                     string* pMsg)
    {
        vector<string> headers;
        if (!getAPIKeyHeader(&headers, pMsg)) return false;
        getUserAgentHeader(&headers);

        string url = apiUrl + "communicate";

        string response;
        if (!Http::post(url, request, "text/icfp", &headers, &response, pMsg))
        {
            return false;
        }

        if (pResponse) *pResponse = std::move(response);
        return true;
    }
}
